import math
from appmodel import AppModel

class AppView:

    def __init__(self, _model=[]):
        self.model = _model

        # Defined colors
        self.colorBackground = [1.00, 1.00, 1.00] # white
        self.colorSelection = [1.00, 0.00, 0.00]  # red
        self.colorPoint = [0.00, 0.00, 0.50]  # dark blue
        self.colorPointSelection = [1.00, 0.00, 0.00]  # red
        self.colorSegment = [0.00, 0.00, 0.00]  # black
        self.colorSegmentSelection = [1.00, 0.00, 0.00]  # red
        self.colorPatch = [0.50, 0.75, 0.50]  # medium green
        self.colorPatchSelection = [1.00, 0.75, 0.75]  # light red
        self.colorCollecting = [1.00, 0.00, 0.00]  # red
        self.colorGrid = [0.00, 0.00, 0.00]  # black
        self.colorSdvPoint = [0.00, 0.00, 0.50]  # dark blue
        self.colorMesh = [0.75, 0.75, 1.00]  # light blue

    def setModel(self, _model):
        self.model = _model

    def getModel(self):
        return self.model

    def isEmpty(self):
        return (self.model is None) or (self.model.isEmpty())

    def getPoints(self):
        return self.model.getVerts()

    def getSegments(self):
        return self.model.getEdges()

    def getPatches(self):
        return self.model.getFaces()

    def isPointSelected(self, _vtx):
        return False

    def isSegmentSelected(self, _seg):
        return False

    def isPatchSelected(self, _ptch):
        return False

    def getPointLoc(self, _vtx):
        return self.model.getVertexPt(_vtx)

    def getSegmentPtsToDraw(self, _seg):
        return self.model.getEdgePtsToDraw(_seg)

    def getPatchPtsToDraw(self, _ptch):
        return self.model.getFacePtsToDraw(_ptch)

    def getBoundBox(self):

        if self.model.isEmpty():
            return 0.0, 10.0, 0.0, 10.0

        verts = self.model.getVerts()
        pt = verts[0]
        x = pt.getX()
        y = pt.getY()

        xmin = x
        ymin = y
        xmax = x
        ymax = y

        for i in range(1, len(verts)):
            pt = verts[i]
            x = pt.getX()
            y = pt.getY()
            xmin = min(x, xmin)
            xmax = max(x, xmax)
            ymin = min(y, ymin)
            ymax = max(y, ymax)

        return xmin, xmax, ymin, ymax

    def getColorBackground(self):
        return self.colorBackground

    def getColorSelection(self):
        return self.colorSelection

    def getColorPoint(self):
        return self.colorPoint

    def getColorPointSelection(self):
        return self.colorPointSelection

    def getColorSegment(self):
        return self.colorSegment

    def getColorSegmentSelection(self):
        return self.colorSegmentSelection

    def getColorPatch(self):
        return self.colorPatch

    def getColorPatchSelection(self):
        return self.colorPatchSelection

    def getColorCollecting(self):
        return self.colorCollecting

    def getColorGrid(self):
        return self.colorGrid

    def getColorSdvPoint(self):
        return self.colorSdvPoint

    def getColorMesh(self):
        return self.colorMesh
