from compgeom.pnt2d import Pnt2D


class CompGeom:

    ABSTOL = 1e-7  # Absolute tolerance value

    # ---------------------------------------------------------------------
    @staticmethod
    def orient2d(pa, pb, pc):
        acx = pa[0] - pc[0]
        bcx = pb[0] - pc[0]
        acy = pa[1] - pc[1]
        bcy = pb[1] - pc[1]
        return acx * bcy - acy * bcx

    # ---------------------------------------------------------------------
    # Return the sign (NEGATIVE, ZERO, or POSITIVE) of the oriented
    # twice area formed by three given points.
    @staticmethod
    def signOrient2d(_p1, _p2, _p3):

        det = 0.0
        pa = [_p1.getX(), _p1.getY()]
        pb = [_p2.getX(), _p2.getY()]
        pc = [_p3.getX(), _p3.getY()]

        det = CompGeom.orient2d(pa, pb, pc)
        if(det != 0.0):
            if(det > 0.0):
                return 'POSITIVE'
            else:
                return 'NEGATIVE'

        return 'ZERO'

    # ---------------------------------------------------------------------
    # Return the signed value of the oriented twice area formed by
    # three given points.
    @staticmethod
    def valOrient2d(_p1, _p2, _p3):
        pa = [_p1.getX(), _p1.getY()]
        pb = [_p2.getX(), _p2.getY()]
        pc = [_p3.getX(), _p3.getY()]

        det = CompGeom.orient2d(pa, pb, pc)
        return det

     # ---------------------------------------------------------------------
   # Return a flag (true or false) stating whether the three given
    # points are collinear.
    @staticmethod
    def areCollinear(_p1,  _p2, _p3):
        return CompGeom.signOrient2d(_p1, _p2, _p3) == 'ZERO'

    # ---------------------------------------------------------------------
    # Return a flag (true or false) stating whether the third given
    # point is on the left side of oriented segment formed by the
    # first given points.
    @staticmethod
    def isLeftSide(_p1, _p2, _p3):
        return CompGeom.signOrient2d(_p1, _p2, _p3) == 'POSITIVE'

     # ---------------------------------------------------------------------
   # Return a flag (true or false) stating whether the third given
    # point is on the right side of oriented segment formed by the
    # first given points.
    @staticmethod
    def isRightSide(_p1, _p2, _p3):
        return CompGeom.signOrient2d(_p1, _p2, _p3) == 'NEGATIVE'

    # ---------------------------------------------------------------------
    # Return the sign (NEGATIVE, ZERO, or POSITIVE) of the oriented
    # twice area formed by three given points.
    # This function uses conventional floating-point operations to
    # compute the area and compares the area result with a hard-coded
    # very small tolerance value (ABSTOL).
    @staticmethod
    def signArea2d(_p1, _p2, _p3):
        det = Pnt2D.area2d(_p1, _p2, _p3)
        if abs(det) < CompGeom.ABSTOL:
            return 'ZERO'
        if det > 0.0:
            return 'POSITIVE'
        return 'NEGATIVE'

    # ---------------------------------------------------------------------
    # Return the signed value of the oriented twice area formed by
    # three given points.
    # This function uses conventional floating-point operations to
    # compute the signed area.
    @staticmethod
    def valArea2d(_p1,  _p2, _p3):
        return Pnt2D.area2d(_p1, _p2, _p3)

     # ---------------------------------------------------------------------
   # Check for collinear segments 'p1'-'p2' and 'p3'-'p4'.
    @staticmethod
    def checkCollinearSegments(_p1, _p2, _p3, _p4):
        sign123 = CompGeom.signArea2d(_p1, _p2, _p3)
        sign124 = CompGeom.signArea2d(_p1, _p2, _p4)

        # Check for collinear segments
        if sign123 == 'ZERO' and sign124 == 'ZERO':
            return True
        return False
